@@ -1,6 +1,7 @@
 #!/usr/bin/env python
 import argparse
 import logging
+import os
 import sys
 from argparse import ArgumentParser, Namespace
 from enum import IntEnum
@@ -71,11 +72,11 @@ def configure_logging():
     logging.root.addHandler(stderr_handler)
 
 
-def existing_directory(val: str) -> Path:
-    directory_path = Path(val)
-    if not directory_path.is_dir():
-        raise argparse.ArgumentTypeError(f"Directory '{val}' doesn't exists")
-    return directory_path
+def existing_file(val: str) -> Path:
+    input_path = Path(val)
+    if not input_path.exists():
+        raise argparse.ArgumentTypeError(f"Path '{val}' doesn't exists")
+    return input_path
 
 
 def nonempty_string(val: str) -> str:
@@ -148,7 +149,8 @@ class _ShowHelp(argparse.Action):
             except TagError as tag_error:
                 parser.exit(ErrorCode.USAGE_ERROR, str(tag_error))
                 return
-            log.info(tag_factory.configuration_signature)
+            log.info("")
+            log.info(indent(tag_factory.configuration_signature, "  "))
             log.info("")
             log.info(tag_factory.short_description)
             if tag_factory.long_description:
@@ -363,9 +365,10 @@ def process_cli_configuration(argv: List[str]) -> RuntimeConfiguration:
         help="Template used to generate new filename/path",
     )
     parser.add_argument(
-        "input_directory",
-        type=existing_directory,
-        help="Input directory where files to rename are stored",
+        "file",
+        type=existing_file,
+        nargs="+",
+        help="Input files or directories where files to rename are stored",
     )
     parser.add_argument(
         "-l",
@@ -428,7 +431,7 @@ def process_cli_configuration(argv: List[str]) -> RuntimeConfiguration:
 
     configuration = RuntimeConfiguration(
         template=args.template,
-        input_directory=args.input_directory,
+        input_paths=args.file,
         recursive=args.recursive,
         include_hidden=args.include_hidden,
         dry_run=args.dry_run,
@@ -520,6 +523,7 @@ class ErrorCode(IntEnum):
 def main() -> int:
     configure_logging()
     argv = sys.argv[1:]
+    original_cwd = os.getcwd()
     try:
         config = process_cli_configuration(argv)
         registry = build_tag_registry()
@@ -547,8 +551,10 @@ def main() -> int:
         log.error(f"Error: {exc}")
         return ErrorCode.INVALID_DESTINATION_ERROR
     except Exception as exc:
-        log.error(f"Unknown error: {exc}")
+        log.error(f"Unknown error: {exc.__class__.__name__} {exc}")
         return ErrorCode.UNKNOWN_ERROR
+    finally:
+        os.chdir(original_cwd)
 
 
 def throwing_main():
