@@ -3,7 +3,7 @@ import os
 from dataclasses import dataclass
 from enum import Enum
 from pathlib import Path
-from typing import Callable, Iterable, Optional, Union
+from typing import Callable, Iterable, List, Optional, Union
 
 from tempren.file_filters import (
     FileFilterInverter,
@@ -15,8 +15,10 @@ from tempren.file_filters import (
 )
 from tempren.file_sorters import TemplateFileSorter
 from tempren.filesystem import (
+    CombinedFileGatherer,
     DestinationAlreadyExistsError,
     DryRunRenamer,
+    ExplicitFileGatherer,
     FileGatherer,
     FileMover,
     FileRenamer,
@@ -69,7 +71,7 @@ ManualConflictResolver = Callable[[Path, Path], Union[ConflictResolutionStrategy
 @dataclass
 class RuntimeConfiguration:
     template: str
-    input_directory: Path
+    input_paths: List[Path]
     recursive: bool = False
     include_hidden: bool = False
     dry_run: bool = False
@@ -94,7 +96,6 @@ def manual_resolver_placeholder(
 
 class Pipeline:
     log: logging.Logger
-    _input_directory: Path
     file_gatherer: FileGatherer
     sorter: Optional[Callable[[Iterable[File]], Iterable[File]]] = None
     path_generator: PathGenerator
@@ -108,19 +109,11 @@ class Pipeline:
             manual_resolver_placeholder
         )
 
-    @property
-    def input_directory(self) -> Path:
-        return self._input_directory
-
-    @input_directory.setter
-    def input_directory(self, input_path: Path):
-        self._input_directory = input_path.absolute()
-
     def execute(self):
         all_files = []
-        self.log.info(f"Gathering paths in {self.input_directory}")
-        os.chdir(self.input_directory)
-        for file in self.file_gatherer.gather_in(self.input_directory):
+
+        self.log.info(f"Gathering paths...")
+        for file in self.file_gatherer.gather_files():
             self.log.debug("Checking %s", file)
             if not self.file_filter(file):
                 self.log.debug("%s filtered out", file)
@@ -139,6 +132,11 @@ class Pipeline:
         # transitional conflicts.
         backlog = []
         for file in all_files:
+            # Renamer will receive a relative path,
+            # to make it valid we need to change the current directory before processing:
+            # TODO: Make renamer operate on File instances instead
+            os.chdir(file.input_directory)
+
             try:
                 self.log.debug("Generating new name for %r", file)
                 new_relative_path = self.path_generator.generate(file)
@@ -248,15 +246,21 @@ def build_pipeline(
     manual_conflict_resolver: ManualConflictResolver,  # TODO: Move to the RuntimeConfiguration
 ) -> Pipeline:
     log.debug("Building pipeline")
-    pipeline = Pipeline()
-    pipeline.input_directory = config.input_directory
     tree_builder = TagTreeBuilder()
+    pipeline = Pipeline()
 
-    if config.recursive:
-        pipeline.file_gatherer = RecursiveFileGatherer()
-    else:
-        pipeline.file_gatherer = FlatFileGatherer()
+    file_gatherers: List[FileGatherer] = []
+    input_directories = filter(lambda p: p.is_dir(), config.input_paths)
+    for directory in input_directories:
+        if config.recursive:
+            file_gatherers.append(RecursiveFileGatherer(directory))
+        else:
+            file_gatherers.append(FlatFileGatherer(directory))
 
+    input_files = list(filter(lambda p: p.is_file(), config.input_paths))
+    if any(input_files):
+        file_gatherers.append(ExplicitFileGatherer(input_files))
+    pipeline.file_gatherer = CombinedFileGatherer(file_gatherers)
     pipeline.file_gatherer.include_hidden = config.include_hidden
 
     def _compile_template(template_text: str) -> Pattern:
